rand('seed',2018);
s = chebfun('s');
n = 400; % sample size
v = 1;
sim_itr = 1000; % number of monte carlo simulation
lambda = 1e-6;

% generating phi_k
phi = chebfun(@(s) 1,[0,1]);
for k = 2:50
    phi_k = chebfun(@(s) cos((k-1)*pi*s),[0,1]);
    phi = [phi;phi_k];
end
phi = phi * sqrt(2);

% generating basis_k
knots = (0:8)/8;
norder = 4;
nbasis = norder + length(knots) - 2;
bobj = create_bspline_basis([0,1],nbasis,norder,knots);
S = eval_penalty(bobj,2);
basis = chebfun(@(s) eval_basis_k(s,bobj,1),[0,1]);
for k = 2:nbasis
    basis = [basis;chebfun(@(s) eval_basis_k(s,bobj,k),[0,1])];
end

% storage of all estimated values of all simulation
theta_all = zeros(sim_itr,nbasis+1);

% theta
theta = 1;

% beta(s)
beta = chebfun('-9./(50-45.*x)-0.9',[0,1]);

cp_Z = 0;
cp = zeros(100,1);
test_stat = zeros(100,sim_itr); % for beta_hat
test_stat_se = zeros(100,sim_itr);

theta_se = zeros(sim_itr); 
theta_hat = zeros(sim_itr);
error_eigen = zeros(sim_itr,1);

cp_LRT = zeros(sim_itr,1);
LRT = zeros(sim_itr,1);

for itr = 1:sim_itr
    
    % generating Z
    Z = normrnd(0,1,n,1);
    
    % generating X(s)
    k = 1;
    U = unifrnd(-3,3,50,1);
    xi = (-1)^(k+1) * k^(-v/2);
    X = xi * U(1) * phi(k);
    for k = 2:50
        X = X + xi * U(k) * phi(k);
    end
    
    for i = 2:n
        k = 1;
        U = unifrnd(-3,3,50,1);
        xi = (-1)^(k+1) * k^(-v/2);
        Xs = xi * U(1) * phi(k);
        for k = 2:50
            Xs = Xs + xi * U(k) * phi(k);
        end
        X = [X;Xs];
    end
    
    eta = zeros(n,1);
    for i = 1:n
        eta(i) = sum(times(X(i),beta));
    end
    eta = eta + theta*Z;
    
    %%%%%%% h0(t) = t %%%%%%%%%
    Tu = wblrnd((3./exp(eta)).^(1/3),3); % failure time
  %  Tc = exprnd(3.9,n,1);     % censoring rate 33%
    Tc = exprnd(15,n,1);       % censoring rate 12%
    delta = Tu<=Tc;            % delta
    Y = min(Tu,Tc);
    1 - mean(delta)            % censoring rate
    
    % constructing script_Y
    script_Y = zeros(n,n);
    for i = 1:n
        script_Y(:,i) = (Y >= Y(i));
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%H1 Unconstrainted%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    theta_init = 0.1;
    theta_start = ones(nbasis+1,1) * theta_init;
    
    int_X_basis = zeros(n,nbasis);
    for j = 1:nbasis
        for i = 1:n
            int_X_basis(i,j) = sum(times(X(i),basis(j)));
        end
    end
    
    theta_H1 = fminunc(@(x) neg_log_likelihood(x,basis,nbasis,delta,lambda,S,X,Z,script_Y,n,int_X_basis),theta_start);
    
    theta_all(itr,:) = theta_H1'; % storing all hat
    sprintf(' itr=%13.5f\n \n',itr )
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Calculate exp(W) for B()
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    theta_Z = theta_H1(1);
    theta_X = theta_H1(2:(nbasis+1));
    
    eta = zeros(n,1);
    for i = 1:nbasis
        eta = eta + int_X_basis(:,i) * theta_X(i);
    end
    eta = eta + theta_Z*Z;
    expW = exp(eta);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    BB = B(Y,expW,delta,Z,n);
    SS = Sigma(Y,expW,delta,Z,n);
    
    theta_SE = sqrt(SS/(BB*BB*n));
    theta_se(itr) = theta_SE; % storing se(theta)
    
    theta_Z = theta_H1(1);  % storing theta_hat
    theta_hat(itr) =  theta_Z;
    if(abs(theta_Z - theta) < 1.96*theta_SE)
        cp_Z = cp_Z + 1;
    end
    
    theta_X = theta_H1(2:(nbasis+1));
    beta_hat = basis(1) * theta_X(1);
    for i = 2:nbasis
        beta_hat = beta_hat + basis(i) * theta_X(i);
    end

    Fst = F_s_t(Y,expW,delta,X,n);
    multi = eigenfunctionCalculation(n,lambda,Fst);
    h = 0.01;
    tt = 0.01:h:1;
    test_stat_1 = chebfun(beta_hat-beta);
    test_stat(:,itr) = test_stat_1(tt) .* multi'; 
    test_stat_se(:,itr) = 1./multi'; % storing beta_hat se
    
    for i = 1:100
        aa = abs(test_stat(i,1:itr))<1.96;
        cp(i) = mean(aa);
    end
    cp' % print the pointwise cp on screen
    
    LRT(itr) = eigenfunctionCalculation_LRT(n,lambda,Fst,theta_H1,basis,nbasis,delta,S,X,Z,script_Y,int_X_basis,beta);
    if(abs(LRT(itr))<1.96)
        cp_LRT(itr) = 1;
        
    end
     sprintf('itr=%13.5f sum(cp_LRT)=%13.5f cp_LRT(itr)/itr=%13.5f LRT(itr)=%13.5f  \n',itr ,sum(cp_LRT),sum(cp_LRT)/itr,LRT(itr) )
    
end

 % finally, plot the average of estimated beta and the true beta
theta_mean= mean(theta_all(:,2:11));
beta_mean = basis(1) * theta_mean(1);
for i = 2:nbasis
    beta_mean = beta_mean + basis(i) * theta_mean(i);
end
plot([beta_mean;beta])

% average of estimated theta
mean(theta_all(:,1))
